@@ -38,7 +38,7 @@ class DummyBuilder(DummyBuilderWithNoPrechunk):
 SERVER_URL = "tcp://127.0.0.1:8234"
 
 
-@pytest.fixture("function")
+@pytest.fixture(scope="function")
 async def master_server(event_loop, log_to_stdout):
 
     task = asyncio.create_task(
@@ -78,7 +78,7 @@ async def test_master_give_out_chunks(master_server, log_to_stdout):
 
 @pytest.mark.asyncio
 async def test_worker():
-    with Pair1(listen=SERVER_URL, polyamorous=True, recv_timeout=100) as worker_socket:
+    with Pair1(listen=SERVER_URL, polyamorous=True, recv_timeout=500) as worker_socket:
 
         worker_task = asyncio.create_task(worker(SERVER_URL, num_workers=1))
 
@@ -119,6 +119,6 @@ async def test_no_prechunk(caplog):
     )
     await asyncio.sleep(1)
     assert (
-        f"Can't distributed process DummyBuilderWithNoPrechunk. Skipping for now"
+        "Can't distributed process DummyBuilderWithNoPrechunk. Skipping for now"
         in caplog.text
     )
